@@ -1,10 +1,11 @@
 import json
 import six
 
+from django.forms import widgets
 from rest_framework import exceptions
 from rest_framework.serializers import *
 
-# Haking rest_framework in order to meet our crazy design specifications
+# CONFINE compatibility layer for DRF
 
 
 class RelHyperlinkedRelatedField(HyperlinkedRelatedField):
@@ -12,18 +13,30 @@ class RelHyperlinkedRelatedField(HyperlinkedRelatedField):
     HyperlinkedRelatedField field providing a relation object rather than flat URL 
     """
     def to_native(self, obj):
-        # FIXME this doesn't work when posting
+        """ 
+        CONFINE specs to DRF compat
+        converts from "http//example.org" to { "uri": "http://example.org" }
+        """
         url = super(RelHyperlinkedRelatedField, self).to_native(obj)
         if url is None:
              return None
-        return {'uri': url}
+        return { 'uri': url }
     
     def from_native(self, value):
-        # TODO this is bullshit, fixit!
+        """ 
+        CONFINE specs to DRF compat
+        converts from { "uri": "http://example.org" } to "http//example.org"
+        """
         if isinstance(value, six.text_type):
             value = value.replace("u'", '"').replace("'", '"')
-            value = json.loads(value)
-        value = value.pop('uri')
+            try:
+                value = json.loads(value)
+            except ValueError:
+                raise exceptions.ParseError("%s is not an object" % value)
+        if isinstance(value, dict):
+            value = value.pop('uri', None)
+        else:
+            raise exceptions.ParseError("%s is not an object" % value)
         return super(RelHyperlinkedRelatedField, self).from_native(value)
 
 
@@ -36,11 +49,13 @@ class UriHyperlinkedModelSerializer(HyperlinkedModelSerializer):
     
 
     def __init__(self, *args, **kwargs):
+        """ url to uri renaming """
         super(UriHyperlinkedModelSerializer, self).__init__(*args, **kwargs)
         self.fields['uri'] = self.fields.pop('url', None)
-   
+
 
 class HyperlinkedFileField(FileField):
+    """ display file url instead of file path """
     def to_native(self, value):
         if value:
             request = self.context.get('request')
@@ -54,7 +69,7 @@ class PropertyField(WritableField):
     need a custom override of restore_object method.
     """
     def to_native(self, value):
-        """ Dict-like representation of a Property Model"""
+        """ dict-like representation of a Property Model"""
         return dict((prop.name, prop.value) for prop in value.all())
     
     def from_native(self, value):
@@ -64,7 +79,7 @@ class PropertyField(WritableField):
         properties = []
         if value:
             model = getattr(parent.opts.model, self.source or 'properties').related.model
-            if isinstance(value, six.text_type):
+            if isinstance(value, basestring):
                 try:
                     value = json.loads(value)
                 except:
@@ -73,6 +88,7 @@ class PropertyField(WritableField):
                 # POST (new parent object)
                 return [ model(name=n, value=v) for n,v in value.iteritems() ]
             # PUT
+            to_save = []
             for (name, value) in value.iteritems():
                 try:
                     # Update existing property
@@ -82,9 +98,36 @@ class PropertyField(WritableField):
                     prop = model(name=name, value=value)
                 else:
                     prop.value = value
+                    to_save.append(prop.pk)
                 properties.append(prop)
-        # TODO this should be done on save()
+        
         # Discart old values
         if related_manager:
-            related_manager.all().delete()
+            for obj in related_manager.all():
+                if obj.pk not in to_save:
+                    # TODO do it in serializer.save() self.object._deleted ?
+                    obj.delete()
         return properties
+
+
+class MultiSelectField(ChoiceField):
+    widget = widgets.CheckboxSelectMultiple
+    
+    def field_from_native(self, data, files, field_name, into):
+        """ convert multiselect data into comma separated string """
+        if field_name in data:
+            data = data.copy()
+            try:
+                # data is a querydict when using forms
+                data[field_name] = ','.join(data.getlist(field_name))
+            except AttributeError:
+                data[field_name] = ','.join(data[field_name])
+        return super(MultiSelectField, self).field_from_native(data, files, field_name, into)
+    
+    def valid_value(self, value):
+        """ checks for each item if is a valid value """
+        for val in value.split(','):
+            valid = super(MultiSelectField, self).valid_value(val)
+            if not valid:
+                return False
+        return True
