@@ -27,19 +27,19 @@ class Reboot(APIView):
     rel = 'http://confine-project.eu/rel/server/do-reboot'
     
     def post(self, request, *args, **kwargs):
-        if request.DATA is None:
+        if not request.DATA:
             node = get_object_or_404(Node, pk=kwargs.get('pk'))
             self.check_object_permissions(self.request, node)
             node.reboot()
             response_data = {'detail': 'Node instructed to reboot'}
-            return Response(response_data, status=status.HTTP_202_ACCEPTED)
-        raise exceptions.ParseError(detail='This endpoint only accepts null data')
+            return Response(response_data, status=status.HTTP_200_OK)
+        raise exceptions.ParseError(detail='This endpoint do not accept data')
 
 
 class RequestCert(APIView):
     """
-    **Relation type:** [`http://confine-project.eu/rel/server/do-request-cert`](
-        http://confine-project.eu/rel/server/do-request-cert)
+    **Relation type:** [`http://confine-project.eu/rel/controller/do-request-cert`](
+        http://confine-project.eu/rel/controller/do-request-cert)
     
     Contains the function URI used to upload this node's certificate request to 
     be signed by the testbed CA and set as the node's certificate.
@@ -47,7 +47,7 @@ class RequestCert(APIView):
     POST data: `ASCII-armored PEM representation of the CSR as a string.`
     """
     url_name = 'request-cert'
-    rel = 'http://confine-project.eu/rel/server/do-request-api-cert'
+    rel = 'http://confine-project.eu/rel/controller/do-request-api-cert'
     
     def post(self, request, *args, **kwargs):
         csr = request.DATA
