@@ -4,13 +4,15 @@ from django.db import models
 
 from controller.models.fields import NullableCharField, NullableTextField
 from controller.settings import PRIV_IPV6_PREFIX, PRIV_IPV4_PREFIX_DFLT, SLIVER_MAC_PREFIX_DFLT
-from controller.core.validators import validate_prop_name, validate_net_iface_name
+from controller.core.validators import validate_name, validate_prop_name, validate_net_iface_name
+from controller.utils.functional import cached
 from controller.utils.singletons.models import SingletonModel
 from pki import ca, Bob
 
 from . import settings
 from .utils import get_mgmt_backend_class
-from .validators import validate_sliver_mac_prefix, validate_ipv4_range, validate_dhcp_range
+from .validators import (validate_sliver_mac_prefix, validate_ipv4_range,
+        validate_dhcp_range, validate_priv_ipv4_prefix)
 
 
 class Node(models.Model):
@@ -46,8 +48,7 @@ class Node(models.Model):
     name = models.CharField(max_length=256, unique=True,
             help_text='A unique name for this node. A single non-empty line of '
                       'free-form text with no whitespace surrounding it.',
-            validators=[validators.RegexValidator('^\w[\s\w.@+-]+\w$',
-                        'Enter a valid name.', 'invalid')])
+            validators=[validate_name])
     cert = NullableTextField('Certificate', unique=True, null=True, blank=True,
             help_text='X.509 PEM-encoded certificate for this RD. The certificate '
                       'may be signed by a CA recognised in the testbed and required '
@@ -92,8 +93,8 @@ class Node(models.Model):
                       'used as the node sliver MAC prefix. See <a href="http://wiki.'
                       'confine-project.eu/arch:addressing">addressing</a> for legal '
                       'values. %s when null.</a>.' % SLIVER_MAC_PREFIX_DFLT)
-    priv_ipv4_prefix = models.GenericIPAddressField('Private IPv4 prefix',
-            protocol='IPv4', null=True, blank=True,
+    priv_ipv4_prefix = NullableCharField('Private IPv4 prefix', null=True,
+            blank=True, max_length=19, validators=[validate_priv_ipv4_prefix],
             help_text='IPv4 /24 network in CIDR notation used as a node private IPv4 '
                       'prefix. See <a href="http://wiki.confine-project.eu/arch:'
                       'addressing">addressing</a> for legal values. %s When null.'
@@ -166,6 +167,7 @@ class Node(models.Model):
         super(Node, self).save(*args, **kwargs)
     
     @property
+    @cached
     def mgmt_net(self):
         return get_mgmt_backend_class()(self)
     
@@ -272,5 +274,6 @@ class Server(SingletonModel):
         return 'Server'
     
     @property
+    @cached
     def mgmt_net(self):
         return get_mgmt_backend_class()(self)
