@@ -21,11 +21,15 @@ from users.helpers import filter_group_queryset
 
 from .actions import renew_selected_slices, reset_selected, update_selected, create_slivers
 from .filters import MySlicesListFilter, MySliversListFilter, SliverSetStateListFilter
-from .forms import SliceAdminForm, SliverIfaceInlineForm, SliverIfaceInlineFormSet, SliceSliversForm
-from .helpers import wrap_action, remove_slice_id
+from .forms import (SliceAdminForm, SliverAdminForm, SliverIfaceInlineForm,
+    SliverIfaceInlineFormSet, SliceSliversForm)
+from .helpers import wrap_action, remove_slice_id, state_value
 from .models import Sliver, SliverProp, SliverIface, Slice, SliceProp, Template
 
 
+# List of ifaces displayed in SliverInline (and subclasses)
+SLIVER_IFACES = ['public6', 'public4', 'management', 'isolated', 'debug']
+
 STATE_COLORS = {
     Slice.REGISTER: 'grey',
     Slice.DEPLOY: 'darkorange',
@@ -54,7 +58,7 @@ def computed_sliver_set_state(sliver):
     state = filter(lambda s: s[0] == state, Slice.STATES)[0][1]
     title = ''
     if effective:
-        title = 'Set state from slice, the sliver state is %s' % sliver_state
+        title = 'Set state from slice, sliver set state is &quot;%s&quot;' % sliver_state
         state += '*'
     return mark_safe('<span style="color:%s;" title="%s">%s</span>' % (color, title, state))
 computed_sliver_set_state.short_description = 'Set state'
@@ -91,6 +95,7 @@ class SliverIfaceInline(PermissionTabularInline):
         return super(SliverIfaceInline, self).get_formset(request, obj=obj, **kwargs)
 
 
+SLIVER_EMPTY_LABEL = "(from slice)"
 class SliverAdmin(ChangeViewActions, ChangeListDefaultFilter, PermissionModelAdmin):
     list_display = [
         '__unicode__', admin_link('node'), admin_link('slice'), computed_sliver_set_state
@@ -113,6 +118,13 @@ class SliverAdmin(ChangeViewActions, ChangeListDefaultFilter, PermissionModelAdm
     change_view_actions = [update_selected]
     default_changelist_filters = (('my_slivers', 'True'),)
     change_form_template = "admin/controller/change_form.html"
+    form = SliverAdminForm # FIXME: ignored but don't know why! Check inheritance
+    
+    class Media:
+        css = {
+             'all': (
+                'slices/css/warning-form.css',)
+        }
     
     def __init__(self, *args, **kwargs):
         """ 
@@ -169,15 +181,24 @@ class SliverAdmin(ChangeViewActions, ChangeListDefaultFilter, PermissionModelAdm
         super(SliverAdmin, self).log_addition(request, object)
     
     def formfield_for_dbfield(self, db_field, **kwargs):
-        """ Make description input widget smaller """
+        """ Make description input widget smaller and update empty label """
         if db_field.name == 'description':
             kwargs['widget'] = forms.Textarea(attrs={'cols': 85, 'rows': 5})
-        if db_field.name == 'template':
+        elif db_field.name == 'template':
+            kwargs['empty_label'] = SLIVER_EMPTY_LABEL
             formfield = self.formfield_for_foreignkey(db_field, **kwargs)
             kwargs['widget'] = LinkedRelatedFieldWidgetWrapper(formfield.widget,
                     db_field.rel, self.admin_site)
         return super(SliverAdmin, self).formfield_for_dbfield(db_field, **kwargs)
     
+    def formfield_for_choice_field(self, db_field, request, **kwargs):
+        """ Update empty label """
+        if db_field.name == 'set_state':
+            blank_choice = (('', SLIVER_EMPTY_LABEL),)
+            kwargs['choices'] = blank_choice + Slice.STATES
+        # TODO update empty_label for exp_data and overlay (FileField)
+        return super(SliverAdmin, self).formfield_for_choice_field(db_field, request, **kwargs)
+    
     def change_view(self, request, object_id, form_url='', extra_context=None):
         """ Linked title """
         sliver = get_object_or_404(Sliver, pk=object_id)
@@ -188,6 +209,15 @@ class SliverAdmin(ChangeViewActions, ChangeListDefaultFilter, PermissionModelAdm
             'header_title': 'Change sliver'
         }
         context.update(extra_context or {})
+        # warn user when sliver.set_state > slice.set_state
+        # Only check on GET (for avoid false/duplicated warnings on POSTing)
+        sliver_state = sliver.set_state
+        slice_state = sliver.slice.set_state
+        if (request.method == 'GET' and
+                state_value(sliver_state) > state_value(slice_state)):
+            msg = "Note: the slice's set state \"%s\" overrides the sliver's \
+                   current set state \"%s\"."% (slice_state, sliver_state)
+            messages.warning(request, msg)
         return super(SliverAdmin, self).change_view(request, object_id,
                 form_url=form_url, extra_context=context)
 
@@ -214,7 +244,7 @@ class NodeListAdmin(NodeAdmin):
         url = reverse('admin:slices_slice_add_sliver', kwargs=kwargs)
         return '<a href="%s">%s<a>' % (url, instance.name)
     add_sliver_link.allow_tags = True
-    add_sliver_link.short_description = 'Add on Node'
+    add_sliver_link.short_description = 'Add on node'
     
     def display_sliver_pub_ipv4_range(self, instance):
         """ Show sliver_pub_ipv4_range on changeliste """
@@ -362,6 +392,13 @@ class SliceSliversAdmin(SliverAdmin):
         self.node_id = sliver.node_id
         context = { 'slice': slice }
         context.update(extra_context or {})
+        # warn user on missconfiguration
+        if request.method == 'GET':
+            if not slice.template.is_active or \
+                sliver.template and not sliver.template.is_active:
+                msg = "The template chosen for this sliver is NOT active. "\
+                    "Please check its configuration."
+                messages.warning(request, msg)
         return super(SliceSliversAdmin, self).change_view(request, object_id,
                 form_url=form_url, extra_context=context)
 
@@ -370,11 +407,11 @@ class SliverInline(PermissionTabularInline):
     """ Show slivers in read only fashion """
     model = Sliver
     max_num = 0
-    fields = ['sliver_link', 'node_link', computed_sliver_set_state]
+    fields = ['sliver_link', 'node_link', computed_sliver_set_state] + SLIVER_IFACES
     readonly_fields = [
         'sliver_link', 'node_link', computed_sliver_set_state, 'sliver_note1',
         'sliver_note2'
-    ]
+    ] + SLIVER_IFACES
     
     class Media:
         css = {
@@ -382,6 +419,22 @@ class SliverInline(PermissionTabularInline):
                 'controller/css/hide-inline-id.css',)
         }
     
+    def public6(self, instance):
+        return instance.interfaces.filter(type='public6').count()
+    
+    def public4(self, instance):
+        return instance.interfaces.filter(type='public4').count()
+    
+    def management(self, instance):
+        return instance.interfaces.filter(type='management').count()
+    management.boolean = True
+    
+    def isolated(self, instance):
+        return instance.interfaces.filter(type='isolated').count()
+    
+    def debug(self, instance):
+        return instance.interfaces.filter(type='debug').count()
+    
     def sliver_note1(self, instance):
         """
         <p>This slice must be saved before creating slivers.
@@ -390,7 +443,7 @@ class SliverInline(PermissionTabularInline):
     sliver_note1.short_description = mark_safe(sliver_note1.__doc__)
     
     def sliver_note2(self, instance):
-        """ <a href="add_sliver" class="addlink"> Add Slivers </a> """
+        """ <a href="add_sliver" class="addlink">Add slivers</a> """
     sliver_note2.short_description = mark_safe(sliver_note2.__doc__)
     
     def get_fieldsets(self, request, obj=None):
@@ -399,7 +452,7 @@ class SliverInline(PermissionTabularInline):
             return [(None, {'fields': ['sliver_note1']})]
         # The slices is registred: display add button in the inline header
         if self.has_change_permission(request, obj, view=False):
-            add_button = 'Slivers <a href="add_sliver">(Add another Sliver)</a>'
+            add_button = 'Slivers <a href="add_sliver">(Add another sliver)</a>'
             self.verbose_name_plural = mark_safe(add_button)
         if not obj.slivers.exists():
             return [(None, {'fields': ['sliver_note2']})]
@@ -413,6 +466,8 @@ class SliverInline(PermissionTabularInline):
     
     def sliver_link(self, instance):
         """ Display sliver change link on the inline form """
+        if not instance.pk:
+            return
         kwargs = {
             'slice_id': instance.slice_id,
             'object_id': instance.id}
@@ -427,8 +482,8 @@ class SliverInline(PermissionTabularInline):
 
 
 class SliverNodeInline(SliverInline):
-    fields = ['sliver_link', 'slice_link', computed_sliver_set_state]
-    readonly_fields = ['sliver_link', 'slice_link', computed_sliver_set_state]
+    fields = ['sliver_link', 'slice_link', computed_sliver_set_state] + SLIVER_IFACES
+    readonly_fields = ['sliver_link', 'slice_link', computed_sliver_set_state] + SLIVER_IFACES
     
     class Media:
         js = ('slices/js/collapsed_sliver_inline.js',)
@@ -436,6 +491,13 @@ class SliverNodeInline(SliverInline):
     def get_fieldsets(self, request, obj=None):
         return super(SliverInline, self).get_fieldsets(request, obj=obj)
     
+    def has_delete_permission(self, request, obj=None):
+        """
+        Disallow deleting slivers from node change_view:
+        Why a node admin can be able of doing that?
+        """
+        return False
+
     def slice_link(self, instance):
         return get_admin_link(instance.slice)
     slice_link.short_description = 'Slice'
@@ -449,6 +511,7 @@ class SlicePropInline(PermissionTabularInline):
     class Media:
         js = ('slices/js/collapsed_properties.js',)
 
+
 class SliceAdmin(ChangeViewActions, ChangeListDefaultFilter, PermissionModelAdmin):
     list_display = [
         'name', 'id', 'vlan_nr', colored_set_state, num_slivers, admin_link('template'),
@@ -541,12 +604,19 @@ class SliceAdmin(ChangeViewActions, ChangeListDefaultFilter, PermissionModelAdmi
         return super(SliceAdmin, self).formfield_for_dbfield(db_field, **kwargs)
 
     def change_view(self, request, object_id, form_url='', extra_context=None):
-        """ Warn user when the slice's group cannot manage slices """
+        """ Warn user on missconfigured slice:
+            - when the slice's group cannot manage slices
+            - when slice template is inactive
+        """
         if request.method == 'GET':
             obj = self.get_object(request, object_id)
             if obj and not obj.group.allow_slices:
                 msg = "The slice group does not have permissions to manage slices"
                 messages.warning(request, msg)
+            if not obj.template.is_active:
+                msg = "The template chosen for this slice is NOT active. "\
+                    "Please check its configuration."
+                messages.warning(request, msg)
         return super(SliceAdmin, self).change_view(request, object_id,
                 form_url=form_url, extra_context=extra_context)
 
