@@ -2,13 +2,13 @@ from __future__ import absolute_import
 
 import datetime
 import json
-from dateutil.relativedelta import relativedelta
 
 from django.contrib import admin
+from django.contrib.admin.util import unquote
 from django.conf.urls import patterns, url
 from django.core.urlresolvers import reverse
-from django.http import HttpResponse
-from django.shortcuts import get_object_or_404
+from django.http import HttpResponse, Http404
+from django.shortcuts import get_object_or_404, render_to_response
 from django.utils import timezone
 from django.utils.safestring import mark_safe
 from pygments import highlight
@@ -32,10 +32,12 @@ from slices.models import Sliver, Slice
 
 from .actions import refresh, refresh_state, show_state
 from .filters import NodeStateListFilter, SliverStateListFilter, FirmwareVersionListFilter
-from .helpers import break_headers, break_lines
-from .models import State, StateHistory, NodeSoftwareVersion
+from .helpers import break_headers, break_lines, get_changes_data
+from .models import NodeSoftwareVersion, State, StateHistory
 from .settings import (STATE_NODE_SOFT_VERSION_URL, STATE_NODE_SOFT_VERSION_NAME,
         STATE_FLAPPING_CHANGES, STATE_FLAPPING_MINUTES)
+from .views import (report_view, slices_view, slivers_view, get_slices_data,
+    get_slivers_data)
 
 
 STATES_COLORS = dict(NODE_STATES_COLORS)
@@ -62,6 +64,8 @@ display_metadata.short_description = 'metadata'
 
 
 def display_data(instance):
+    if not instance.data:
+        return ''
     style = '<style>code,pre {font-size:1.13em;}</style><p></p>'
     data = json.dumps(json.loads(instance.data), indent=4, ensure_ascii=False)
     data = break_lines(data)
@@ -168,7 +172,7 @@ class StateHistoryAdmin(admin.ModelAdmin):
         return False
     
     def data_view(self, request, object_id):
-        history = get_object_or_404(StateHistory, pk=object_id)
+        history = get_object_or_404(State, pk=object_id)
         data = {
             'metadata': display_metadata(history),
             'data': display_data(history)
@@ -177,50 +181,7 @@ class StateHistoryAdmin(admin.ModelAdmin):
     
     def changes_view(self, request, object_id):
         state = get_object_or_404(State, pk=object_id)
-        history = state.history
-        now = timezone.now()
-        delta = relativedelta(months=+1)
-        final = datetime.datetime(year=now.year, month=now.month+1, day=1, tzinfo=timezone.utc)
-        monthly = {}
-        distinct_states = set()
-        # Get monthly changes
-        for m in range(1, 13):
-            initial = final-delta
-            changes = history.filter(start__lt=final, end__gt=initial)
-            if not changes:
-                break
-            states = {}
-            for value, start, end in changes.values_list('value', 'start', 'end'):
-                distinct_states = distinct_states.union(set((value,)))
-                if start < initial:
-                    start = initial
-                if end > final:
-                    end = final
-                duration = int((end-start).total_seconds())
-                states[value] = states.get(value, 0)+duration
-            monthly[initial.strftime("%B")] = states
-            final = initial
-        # Fill missing states
-        data = {}
-        for month, duration in monthly.iteritems():
-            current_states = set()
-            for state in duration:
-                data.setdefault(state, []).append(duration[state])
-                current_states = current_states.union(set((state,)))
-            for missing in distinct_states-current_states:
-                data.setdefault(missing, []).append(0)
-        # Construct final data structure
-        series = []
-        for state in data:
-            series.append({
-                'name': state,
-                'data': data[state],
-                'color': STATES_COLORS.get(state, None)
-            })
-        data = {
-            'categories': list(monthly.keys()),
-            'series': series
-        }
+        data = get_changes_data(state)
         return HttpResponse(json.dumps(data), content_type="application/json")
 
 
@@ -257,6 +218,22 @@ class StateAdmin(ChangeViewActions, PermissionModelAdmin):
             url("^(?P<object_id>\d+)/history/$",
                 wrap_admin_view(self, history_admin.changelist_view),
                 name='state_history'),
+            # URLs not wrapped by admin_view for allow anonymous users
+            url("^report/$",
+                report_view,
+                name='state_report'),
+            url("^slices/$",
+                slices_view,
+                name='state_slices'),
+            url("^slices.json$",
+                get_slices_data,
+                name='state_slices_data'),
+            url("^slivers/$",
+                slivers_view,
+                name='state_slivers'),
+            url("^slivers.json$",
+                get_slivers_data,
+                name='state_slivers_data'),
         )
         return urls + super(StateAdmin, self).get_urls()
     
@@ -295,7 +272,9 @@ class StateAdmin(ChangeViewActions, PermissionModelAdmin):
         return False
     
     def change_view(self, request, object_id, form_url='', extra_context=None):
-        state = get_object_or_404(self.model, pk=object_id)
+        state = self.get_object(request, unquote(object_id))
+        if state is None:
+            raise Http404('State with id "%s" does not exists' % object_id)
         context = {
             'title': 'State',
             'obj_opts': state.content_object._meta,
